#charset "us-ascii"
#include <adv3.h>
#include <en_us.h>

/*

actList v. 1.0 Copyright (c) Steve Breslin, 2004.

Thanks to the v-space discussion group for brainstorming this project.

License:

You can use this material however you want, but if you decide to
publicize anything that uses, #include's or otherwise borrows from
this material, you have to make your improvements publicly available,
and advertise them to the IF community.

That way, you will help keep this up to date for everybody else, and
everybody else will help keep it up to date for you.

You may redistribute this verbatim or in modified form only if you
keep the copyright and license intact.

Feel encouraged to release your source code along with your game,
though this isn't a requirement.

********************************************************************

Introduction:

This is an English-only extension, which lists actors in a nicer
format than does the standard library.

This is particularly nice when there are many actors in several
postures and sub-locations. And this is especially needed when you
are using "vantage rooms."

If you are using vantage rooms, be sure to set the locDesc on all
rooms which can be seen from a distance.

Here's a little explanation of vantage rooms, in case you're
interested:

To create a vantage-room, simply use a DistanceConnector to link each
of the surrounding rooms to the vantage room. Surrounding rooms can be
other vantage-rooms. (This works all by itself; use of MultiLoc is not
necessary.)

You can use a MultiLoc as a proxy-object for redirecting commands
intended for the target room (such as "examine tower"). E.g., make a
MultiLoc object "towerProxy" to handle commands directed at the tower
room, when the player is standing in one of the surrounding rooms.
Further, set the proxy object to sightSize=small, so that when
you're in the tower, for example, you won't be able to direct commands
at the tower proxy object. This isn't really useful unless you're
assigning vocabWords to rooms, which isn't a bad idea if you're
implementing multi-room spaces.

At this and for other reasons, you will sometimes want to provide rooms
with vocabWords, for use with 'go to' commands, for example. With this,
in combination with proxy objects designed to handle certain other
commands, bad disambiguation questions can come when the game tries to
figure out whether the room or its proxy-object is intended. This can
be solved by modifying the room so that its logical rank is lower than
that of the proxy object.

*/

;

/***************************************************************/

// some minor additions/modifications:

modify lying
    desc = 'lying down'
;
modify sitting
    desc = 'sitting'
;
modify standing
    desc = 'standing'
;

modify Thing
    locDesc() {
        if (actorInPrep && destName)
            return (actorInPrep + ' ' + destName);
        return nil;
    }
;

modify NestedRoom
    locDesc = simpleLocDesc + ' ' + location.locDesc
    simpleLocDesc = actorInPrep + ' ' + theName
;

/* actlist, take ii:
*/

modify Actor
    actorHereListWith = [ActlistActorGrouper]
;

ActlistActorGrouper: ListGroup
    minGroupSize = 1
    showGroupList(pov, lister, lst, options, indent, infoTab) {
        local actorsCur = [];
        local actorsThere = lst;
        foreach(local obj in lst) {
            if (infoTab[obj] == nil)
                actorsThere -= obj; // filter non-visible items in list
            else if (pov.getOutermostRoom == obj.getOutermostRoom) {
                actorsCur += obj;
                actorsThere -= obj;
            }
        }
        "\n";
        roomList(actorsCur); // first list actors in the pov's room
        "\n";
        while (actorsThere.length()) {
            actorsCur = [actorsThere[1]];
            actorsThere -= actorsCur;
            foreach (local obj in actorsThere) {
                if (obj.getOutermostRoom == actorsCur[1].getOutermostRoom) {
                    actorsCur += obj;
                    actorsThere -= obj;
                }
            }
            roomList(actorsCur); // on a per-room basis
        }
    }

    roomList(actors) {
        local groups = [];
        if (!actors.length) return;
        "\^";

        /* sort the actors into groups, by posture & sub-location */
        foreach(local obj in actors) {
            if (!groups.length)
                groups += [[obj]];
            else {
                local inGroup = nil;
                for(local i = 1; i <= groups.length ; i++) {
                    if (obj.location == groups[i][1].location &&
                        obj.posture == groups[i][1].posture) {
                        groups[i] += obj;
                        inGroup = true;
                        break;
                    }
                }
                if (!inGroup)
                    groups += [[obj]];
            }
        }

if(nil) {} // this just allows the 'else' statement, below, in case
           // TWO_GROUP is not defined.

// if you want the special two-group lister, define TWO_GROUP thus:
#define TWO_GROUP
// if you want to just use the main lister, remove this #define

#ifdef TWO_GROUP

        /* if we have only one or two groups, we'll use this format
         * for listing the actors:
         * <group1> is/are <posture> <locDesc>[, and <group2> is/are
         * <posture?> nearby, <posture?> <locDesc>].
         */
        if (groups.length < 3) {
            local act1; // local variable for any actor in groups[1]
            act1 = groups[1][1];

            /* reverse the groups, if the first group is not directly
             * in the outermost room. (if neither are, no matter: we'll
             * reverse them anyway.)
             */
            if (groups.length == 2 &&
                act1.location != act1.getOutermostRoom) {
                groups = [groups[2],groups[1]];
                act1 = groups[1][1]; // act1 needs to be reset
            }

            "\^<<simpleList(groups[1])>>
            <<groups[1].length > 1 ? 'are' : 'is'>> ";
            if (act1.location != act1.getOutermostRoom ||
                act1.posture != act1.location.defaultPosture) {
                "<<act1.posture.desc>> ";
            }
            if (act1.getOutermostRoom == gPlayerChar.getOutermostRoom)
                "here <<act1.location.locDesc>>";
            else
                "<<act1.location.locDesc>>";

            if (groups.length > 1) {
                local act2 = groups[2][1]; // any actor in groups[2]
                ", and <<simpleList(groups[2])>>
                <<groups[2].length > 1 ? 'are' : 'is'>> ";
                if (act2.location == act1.location)
                    "<<act2.posture.desc>> ";
                "nearby";
                if (act2.location != act1.location)
                    ", <<act2.posture.desc>>";
                if (act2.location != act2.getOutermostRoom)
                    " <<act2.location == act2.getOutermostRoom ?
                    act2.location.locDesc : act2.location.simpleLocDesc>>";
            }
            ".";
        }
#endif

        /* if we have three or more groups, we'll use this format:
         * <all actors> is/are <locDesc>. <group1?> is <posture?>
         * <sub-locDesc?>, <group2?> is/are <posture?>
         * <sub-locDesc?>, ....
         */
        else {
            /* First, list all the actors, and say they are either
             * 'here' (in the PC's location) or provide their
             * location's locDesc.
             */
            "\^<<simpleList(actors)>>
            <<actors.length > 1 ? 'are' : 'is'>>
            <<actors[1].getOutermostRoom ==
              gPlayerChar.getOutermostRoom ?
              'here' : actors[1].getOutermostRoom.locDesc>>. ";

            /* Then, list each group with their posture / sub-location
             * separately (unless they are in the outermost room
             * of the location, and assuming the default posture).
             */
            foreach (local gr in groups) {
                local act = gr[1];
                if (act.location != act.getOutermostRoom ||
                    act.posture != act.location.defaultPosture) {
                    "\^<<simpleList(gr)>>
                    <<gr.length > 1 ? 'are' : 'is'>>
                    <<act.posture.desc>>";
                    if (act.location != act.getOutermostRoom) {
                        " <<act.location.simpleLocDesc>>";
                    }
                ". ";
                }
            }
        }
        "\n";
    }

    simpleList(actors) {
        plainActorLister.showListAll(actors, 0, 0);
    }
;

