#charset "us-ascii"
#include <adv3.h>
#include <en_us.h>

// really we'd want to call this file events.t, but that would create
// confusion with the adv3 module of the same name.

//--!! lock the keep earlier! Geoffrey's entry is triggering the
//==!! lock-the-keep mechanism.

modify BasicLocation
    episodeDaemon() { }
    episodeDaemonID = nil
    startEpisodeDaemon() {
        if(episodeDaemonID == nil)
            episodeDaemonID = new Daemon(self, &episodeDaemon, 1);
    }
    stopEpisodeDaemon() {
        if(episodeDaemonID != nil) {
            episodeDaemonID.removeEvent();
            episodeDaemonID = nil;
        }
    }
;

/* We set up an InitObject to startup the episodeDaemons on the
 * locations that need them
 */
 
InitObject 
    locList = [keep, chamber]
    execute() {
        foreach(local loc in locList)
            loc.startEpisodeDaemon();
    }
;

modify DownLadder
    construct(ladder_) {
        inherited(ladder_);
        if (ladder_.location is in (battlefield, ditch)) {
            local toppler = nil;
            callWithSenseContext(geoffrey, sound, {:"Geoffrey smiles
            grimly, and grips his blade. <q>They are
            weak; the keep will fall!</q>\n"});
            if (mainGate.hasBeenOpened)
                return;
            for (local obj=firstObj(Rebel) ; obj ;
                 obj = nextObj(obj , Rebel)) {
                if (obj.isIn(ladder_.upperEndLoc)) {
                    obj.myParam = ladder_;
                    obj.myPlan = rPushLadder;
                    toppler = obj;
                }
            }
            callWithSenseContext(ranulf, sound, {:"Ranulf
            calls<<toppler ? ' to ' + toppler.theName : ''>>,
            <q>Topple that ladder!</q>\n"});
            // reset all daemons.
            setAnimateDaemons.execute();
        }
    }
    moveInto(newDest) {
        if (newDest == nil) {
            local pusherFound = nil;
            for (local obj=firstObj(Rebel) ; obj ;
                 obj = nextObj(obj, Rebel)) {
                if (obj.myPlan == rPushLadder && obj.myParam == self) {
                    obj.myParam = obj.myEnemy;
                    obj.myPlan = rFindAndAttack;
                    pusherFound = true;
                }
            }
            if (pusherFound) {
                callWithSenseContext(ranulf, sound, {:"Ranulf yells,
                <q>Your meager attempts to break this stronghold are
                no match for our forces!</q>\n"});
                // reset all daemons.
                setAnimateDaemons.execute();
            }
        }
        inherited(newDest);
    }
;

rPushLadder: rStep
    rTrue(a, p) { return !p.location; }
    rPlans(a, p) {
        return [[rBe, rIn, p.location, rDo, rPushLadder, nil]];
    }
    rAction(a, p) {
        nestedActorAction(a, Push, p);
    }
;

modify seigeTower
    makePulled(pulled) {
        inherited(pulled);
        if (distanceToWall == 0) {
            callWithSenseContext(geoffrey, sound, {:"<q>Swordsmen --
            the gatehouse!</q> cries Geoffrey,
            <q>Make open that gate at any cost! Archers, lay
            cover, but remember, I want Ranulf kept alive, and I shall
            flay any bowman who strikes him! -- a fine trophy
            <i>he'll</i> make.</q>"});
        }
    }
;

modify gatehouse
    enteringRoom(traveler) {
        if(traveler.ofKind(Royalist) && !gateAnnouncement) {
            if (!mainGate.hasBeenOpened) {
                callWithSenseContext(ranulf, sound, {:"<q>Archers,
                    back!</q> Ranulf commands, <q>The rest of you,
                stop them opening the gate at all costs!</q>\n"});
                for (local obj=firstObj(RebelArcher) ; obj ;
                     obj = nextObj(obj, RebelArcher)) {
                    if (obj.location not in (bellTower, stableRoof,
                                             kitchenRoof)) {
                        if (rand(2))
                            obj.myParam = courtyardNorth;
                        else if (rand(2))
                            obj.myParam = stableRoof;
                        else 
                            obj.myParam = kitchenRoof;
                        obj.myPlan = rIn;
                        obj.callAnimateDaemon();
                    }
                }
                for (local obj=firstObj(RebelManAtArms) ; obj ;
                     obj = nextObj(obj,RebelManAtArms)) {
                    obj.myParam = myEnemy;
                    obj.myPlan = rFindAndAttack;
                    obj.callAnimateDaemon();
                }
            }
        gateAnnouncement = true;
        }
    }
    gateAnnouncement = nil
;

modify mainGate
    makeOpen(stat) {
        inherited(stat);
        if (hasBeenOpened)
            return;
        hasBeenOpened = true;
        callWithSenseContext(geoffrey, sound, {:"Geoffrey cries,
        <q>We've gained the gate! Seize the keep!</q><.p>"});
        for (local obj = firstObj(Royalist) ; obj ;
             obj = nextObj(obj, Royalist)) {
            obj.myParam = obj.myEnemy;
            obj.myPlan = rFindAndAttack;
        }
        callWithSenseContext(ranulf, sound, {:"Ranulf yells,
        <q>They've taken the wall! Fall back to the
        courtyard! Archers to the keep's balcony!</q>\n"});
        for (local obj=firstObj(Rebel) ; obj ;
             obj = nextObj(obj, Rebel)) {

            if (obj.ofKind(RebelArcher))
                obj.myParam = balcony;
            else
                obj.myParam = courtyardNorth;
            obj.myPlan = rIn;
        }
        ranulf.myParam = balcony;
        ranulf.myPlan = rIn;
        ranulf.rapAct(rIn, balcony);

        callWithSenseContext(geoffrey, sound, {:"<q>The gate is
        opened!</q> cries Geoffrey, <q>Archers, take
        positions; the rest of you, follow me!</q>\n"});
        for (local obj=firstObj(Royalist) ; obj ;
             obj = nextObj(obj, Royalist)) {

            if (obj.ofKind(RoyalistArcher)) {
                obj.myPlan = rIn;
                if (rand(2))
                    obj.myParam = northWall;
                else if (rand(2))
                    obj.myParam = kitchenRoof;
                else
                    obj.myParam = stableRoof;
            }
            else if (obj == geoffrey) {
                obj.myPlan = rIn;
                obj.myParam = courtyardNorth;
            }
            else {
                obj.myPlan = rFindAndAttack;
                obj.myParam = obj.myEnemy;
            }
        }
        
        // now that goals have been reset, now reinitialize all
        // RAP-Actors' daemons.
        setAnimateDaemons.execute();
    }
    hasBeenOpened = nil
;

modify balcony
    enteringRoom(traveler) {
        if(traveler.ofKind(Rebel)) {
            traveler.myPlan = rAmbush;
            traveler.myParam = traveler.myEnemy;
            traveler.callAnimateDaemon();
        }
    }
;

modify Rebel
    NPCDies() {
        local cnt;
        for(local obj=firstObj(Rebel) ; obj ;
                  obj = nextObj(obj,Rebel)) {
            if (obj.location)
                cnt++;
        }
        if (cnt < 5) {
            callWithSenseContext(geoffrey, sound, {:"Geoffrey grins
            grimly, <q>The keep is all but mine. All men to the
            inner keep!</q>\n"});
            for(local obj=firstObj(Royalist) ; obj ;
                      obj = nextObj(obj,Royalist)) {
                obj.myPlan = rIn;
                obj.myParam = chamber;
            }
        }
        else if (cnt < 9) {
            callWithSenseContext(ranulf, sound, {:"Ranulf paces
            nervously. <q>They make gains -- all men fall
            back to the keep!<\q>\n"});
            for(local obj=firstObj(RebelManAtArms) ; obj ;
                      obj = nextObj(obj,RebelManAtArms)) {
                obj.myPlan = rIn;
                obj.myParam = keep;
            }
            for(local obj=firstObj(RebelArcher) ; obj ;
                      obj = nextObj(obj,RebelArcher)) {
                obj.myPlan = rIn;
                obj.myParam = balcony;
            }
        }
        // now that goals have been reset, now reinitialize all
        // RAP-Actors' daemons.
        setAnimateDaemons.execute();
        inherited();
    }
;

modify keep
    enteringRoom(traveler) {
        if (!traveler.ofKind(Royalist) || keepBeingLocked) return;

        callWithSenseContext(ranulf, sound, {:"Ranulf cries <q>They
          enter the keep! Bar the door!</q>\n"});

        // if there's any rebel in the "keep" room, set his plan to
        // lock the door. otherwise, set all rebels' plans to lock
        // the door.
        for (local obj = firstObj(Rebel) ; obj ;
                   obj = nextObj(obj, Rebel)) {
            if (obj.location == self) {
                obj.myParam = keepDoor;
                obj.myPlan = rLocked;
                keepBeingLocked = true;
            }
        }
        if (!keepBeingLocked) {
            for (local obj = firstObj(Rebel) ; obj ;
                       obj = nextObj(obj, Rebel)) {
                obj.myParam = keepDoor;
                obj.myPlan = rLocked;
                keepBeingLocked = true;
            }
        }

        callWithSenseContext(geoffrey, sound, {:"<q>They're soon
        barring the keep door,</q> remarks the Baron.
        <q>Secure and search the outlying buildings: <i>Adelicia must
        not be harmed</i>! Archers to forward positions"});
            foreach(local obj in bellTower.contents) {
                if (obj.ofKind(Rebel)) {
                    callWithSenseContext(geoffrey, sound, {:" -- and
                    check that blasted belfrey!</q>\n
                    A man-at-arms reports: <q>The stairway to the
                    belfrey lies --</q>\n
                    Geoffrey scowls, <q>I care not for excuses; give
                    me answers"});
                    break;
                }
            }
        callWithSenseContext(geoffrey, sound, {:"!</q>\n"});
        for (local obj=firstObj(RoyalistArcher) ; obj ;
                   obj = nextObj(obj,RoyalistArcher)) {
            if (obj.location not in (stableRoof, bellTower)) {
                obj.myPlan = rIn;
                if (rand(2))
                    obj.myParam = stableRoof;
                else obj.myParam = bellTower;
            }
        }

        // now that goals have been reset, now reinitialize all
        // RAP-Actors' daemons.
        setAnimateDaemons.execute();
     }
    keepBeingLocked = nil
;

modify keepDoor
    makeLocked(stat) {
        inherited(stat);
        if (lockedBefore)
            return;
        lockedBefore = true;
        for (local obj=firstObj(Rebel) ; obj ;
                   obj = nextObj(obj,Rebel)) {
            obj.myParam = obj.myEnemy;
            obj.myPlan = rAmbush;
        }
        // reset all daemons.
        setAnimateDaemons.execute();
    }
    lockedBefore = nil
;

modify chamber
    enteringRoom(traveler) {
        if(traveler == geoffrey) {
            callWithSenseContext(geoffrey, sound, {:"<q>My
            daughter?</q> cries the Baron, <q>Adelicia!</q>\n"});
            geoffrey.myParam = tower;
            geoffrey.myPlan = rIn;
            geoffrey.callAnimateDaemon();
        }

        if(traveler.ofKind(Royalist) && !adeliciaMoved) {
            callWithSenseContext(traveler, sound, {:"The old soldier
            takes a
            look down the stairs at <<traveler.the_Name>> approaching,
            and turns to Adelicia. <q>Right, my lady, it's time we
            moved you from here.</q> He turns to his younger companion,
            <q>You, take up the rear!</q>\b
            \^<<traveler.the_Name>> tries to fight his way to Adelicia,
            but the younger soldier fights him off while the older one
            drags her up the stairs, while she looks anxiously back at
            <<traveler.the_Name>>.\n"});
            newActorAction(oldSoldier, Climb, chamberStair);
            newActorAction(adelicia, Climb, chamberStair);
            newActorAction(youngSoldier, Climb, chamberStair);
            ranulf.moveIntoForTravel(tower);
            callWithSenseContext(ranulf, sound, {:"Christ's wounds! Blast it, I'll have out with you!\b"});
            newActorAction(ranulf, Climb, chamberStair);
            adeliciaMoved = true;
            ranulf.myParam = tower;
        }
    }
    adeliciaMoved = nil
;   

